@@ -1,26 +1,32 @@
+use std::convert::From;
 use std::fmt::{self, Debug, Display, Formatter};
 use std::ops::{Index, IndexMut};
+use unicode_width::UnicodeWidthChar;
 
 use zellij_utils::vte::ParamsIter;
 
 use crate::panes::alacritty_functions::parse_sgr_color;
+use zellij_tile::data::PaletteColor;
 
 pub const EMPTY_TERMINAL_CHARACTER: TerminalCharacter = TerminalCharacter {
     character: ' ',
     width: 1,
-    styles: CharacterStyles {
-        foreground: Some(AnsiCode::Reset),
-        background: Some(AnsiCode::Reset),
-        strike: Some(AnsiCode::Reset),
-        hidden: Some(AnsiCode::Reset),
-        reverse: Some(AnsiCode::Reset),
-        slow_blink: Some(AnsiCode::Reset),
-        fast_blink: Some(AnsiCode::Reset),
-        underline: Some(AnsiCode::Reset),
-        bold: Some(AnsiCode::Reset),
-        dim: Some(AnsiCode::Reset),
-        italic: Some(AnsiCode::Reset),
-    },
+    styles: RESET_STYLES,
+};
+
+pub const RESET_STYLES: CharacterStyles = CharacterStyles {
+    foreground: Some(AnsiCode::Reset),
+    background: Some(AnsiCode::Reset),
+    strike: Some(AnsiCode::Reset),
+    hidden: Some(AnsiCode::Reset),
+    reverse: Some(AnsiCode::Reset),
+    slow_blink: Some(AnsiCode::Reset),
+    fast_blink: Some(AnsiCode::Reset),
+    underline: Some(AnsiCode::Reset),
+    bold: Some(AnsiCode::Reset),
+    dim: Some(AnsiCode::Reset),
+    italic: Some(AnsiCode::Reset),
+    link_anchor: Some(LinkAnchor::End),
 };
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
@@ -32,6 +38,15 @@ pub enum AnsiCode {
     ColorIndex(u8),
 }
 
+impl From<PaletteColor> for AnsiCode {
+    fn from(palette_color: PaletteColor) -> Self {
+        match palette_color {
+            PaletteColor::Rgb((r, g, b)) => AnsiCode::RgbCode((r, g, b)),
+            PaletteColor::EightBit(index) => AnsiCode::ColorIndex(index),
+        }
+    }
+}
+
 #[derive(Clone, Copy, Eq, PartialEq, Debug)]
 pub enum NamedColor {
     Black,
@@ -54,48 +69,52 @@ pub enum NamedColor {
 
 impl NamedColor {
     fn to_foreground_ansi_code(self) -> String {
-        match self {
-            NamedColor::Black => format!("{}", 30),
-            NamedColor::Red => format!("{}", 31),
-            NamedColor::Green => format!("{}", 32),
-            NamedColor::Yellow => format!("{}", 33),
-            NamedColor::Blue => format!("{}", 34),
-            NamedColor::Magenta => format!("{}", 35),
-            NamedColor::Cyan => format!("{}", 36),
-            NamedColor::White => format!("{}", 37),
-            NamedColor::BrightBlack => format!("{}", 90),
-            NamedColor::BrightRed => format!("{}", 91),
-            NamedColor::BrightGreen => format!("{}", 92),
-            NamedColor::BrightYellow => format!("{}", 93),
-            NamedColor::BrightBlue => format!("{}", 94),
-            NamedColor::BrightMagenta => format!("{}", 95),
-            NamedColor::BrightCyan => format!("{}", 96),
-            NamedColor::BrightWhite => format!("{}", 97),
-        }
+        let v = match self {
+            NamedColor::Black => 30,
+            NamedColor::Red => 31,
+            NamedColor::Green => 32,
+            NamedColor::Yellow => 33,
+            NamedColor::Blue => 34,
+            NamedColor::Magenta => 35,
+            NamedColor::Cyan => 36,
+            NamedColor::White => 37,
+            NamedColor::BrightBlack => 90,
+            NamedColor::BrightRed => 91,
+            NamedColor::BrightGreen => 92,
+            NamedColor::BrightYellow => 93,
+            NamedColor::BrightBlue => 94,
+            NamedColor::BrightMagenta => 95,
+            NamedColor::BrightCyan => 96,
+            NamedColor::BrightWhite => 97,
+        };
+
+        v.to_string()
     }
     fn to_background_ansi_code(self) -> String {
-        match self {
-            NamedColor::Black => format!("{}", 40),
-            NamedColor::Red => format!("{}", 41),
-            NamedColor::Green => format!("{}", 42),
-            NamedColor::Yellow => format!("{}", 43),
-            NamedColor::Blue => format!("{}", 44),
-            NamedColor::Magenta => format!("{}", 45),
-            NamedColor::Cyan => format!("{}", 46),
-            NamedColor::White => format!("{}", 47),
-            NamedColor::BrightBlack => format!("{}", 100),
-            NamedColor::BrightRed => format!("{}", 101),
-            NamedColor::BrightGreen => format!("{}", 102),
-            NamedColor::BrightYellow => format!("{}", 103),
-            NamedColor::BrightBlue => format!("{}", 104),
-            NamedColor::BrightMagenta => format!("{}", 105),
-            NamedColor::BrightCyan => format!("{}", 106),
-            NamedColor::BrightWhite => format!("{}", 107),
-        }
+        let v = match self {
+            NamedColor::Black => 40,
+            NamedColor::Red => 41,
+            NamedColor::Green => 42,
+            NamedColor::Yellow => 43,
+            NamedColor::Blue => 44,
+            NamedColor::Magenta => 45,
+            NamedColor::Cyan => 46,
+            NamedColor::White => 47,
+            NamedColor::BrightBlack => 100,
+            NamedColor::BrightRed => 101,
+            NamedColor::BrightGreen => 102,
+            NamedColor::BrightYellow => 103,
+            NamedColor::BrightBlue => 104,
+            NamedColor::BrightMagenta => 105,
+            NamedColor::BrightCyan => 106,
+            NamedColor::BrightWhite => 107,
+        };
+
+        v.to_string()
     }
 }
 
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy, Debug, PartialEq, Default)]
 pub struct CharacterStyles {
     pub foreground: Option<AnsiCode>,
     pub background: Option<AnsiCode>,
@@ -108,24 +127,7 @@ pub struct CharacterStyles {
     pub bold: Option<AnsiCode>,
     pub dim: Option<AnsiCode>,
     pub italic: Option<AnsiCode>,
-}
-
-impl Default for CharacterStyles {
-    fn default() -> Self {
-        Self {
-            foreground: None,
-            background: None,
-            strike: None,
-            hidden: None,
-            reverse: None,
-            slow_blink: None,
-            fast_blink: None,
-            underline: None,
-            bold: None,
-            dim: None,
-            italic: None,
-        }
-    }
+    pub link_anchor: Option<LinkAnchor>,
 }
 
 impl CharacterStyles {
@@ -176,6 +178,10 @@ impl CharacterStyles {
         self.strike = strike_code;
         self
     }
+    pub fn link_anchor(mut self, link_anchor: Option<LinkAnchor>) -> Self {
+        self.link_anchor = link_anchor;
+        self
+    }
     pub fn clear(&mut self) {
         self.foreground = None;
         self.background = None;
@@ -188,153 +194,78 @@ impl CharacterStyles {
         self.bold = None;
         self.dim = None;
         self.italic = None;
+        self.link_anchor = None;
     }
     pub fn update_and_return_diff(
         &mut self,
         new_styles: &CharacterStyles,
         changed_colors: Option<[Option<AnsiCode>; 256]>,
     ) -> Option<CharacterStyles> {
-        let mut diff: Option<CharacterStyles> = None;
+        if self == new_styles && changed_colors.is_none() {
+            return None;
+        }
 
-        if new_styles.foreground == Some(AnsiCode::Reset)
-            && new_styles.background == Some(AnsiCode::Reset)
-            && new_styles.strike == Some(AnsiCode::Reset)
-            && new_styles.hidden == Some(AnsiCode::Reset)
-            && new_styles.reverse == Some(AnsiCode::Reset)
-            && new_styles.fast_blink == Some(AnsiCode::Reset)
-            && new_styles.slow_blink == Some(AnsiCode::Reset)
-            && new_styles.underline == Some(AnsiCode::Reset)
-            && new_styles.bold == Some(AnsiCode::Reset)
-            && new_styles.dim == Some(AnsiCode::Reset)
-            && new_styles.italic == Some(AnsiCode::Reset)
-        {
-            self.foreground = Some(AnsiCode::Reset);
-            self.background = Some(AnsiCode::Reset);
-            self.strike = Some(AnsiCode::Reset);
-            self.hidden = Some(AnsiCode::Reset);
-            self.reverse = Some(AnsiCode::Reset);
-            self.fast_blink = Some(AnsiCode::Reset);
-            self.slow_blink = Some(AnsiCode::Reset);
-            self.underline = Some(AnsiCode::Reset);
-            self.bold = Some(AnsiCode::Reset);
-            self.dim = Some(AnsiCode::Reset);
-            self.italic = Some(AnsiCode::Reset);
-            return Some(*new_styles);
-        };
+        if *new_styles == RESET_STYLES {
+            *self = RESET_STYLES;
+            return Some(RESET_STYLES);
+        }
+
+        // create diff from all changed styles
+        let mut diff = CharacterStyles::new();
 
         if self.foreground != new_styles.foreground {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.foreground(new_styles.foreground));
-                self.foreground = new_styles.foreground;
-            } else {
-                diff = Some(CharacterStyles::new().foreground(new_styles.foreground));
-                self.foreground = new_styles.foreground;
-            }
+            diff.foreground = new_styles.foreground;
         }
         if self.background != new_styles.background {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.background(new_styles.background));
-                self.background = new_styles.background;
-            } else {
-                diff = Some(CharacterStyles::new().background(new_styles.background));
-                self.background = new_styles.background;
-            }
+            diff.background = new_styles.background;
         }
         if self.strike != new_styles.strike {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.strike(new_styles.strike));
-                self.strike = new_styles.strike;
-            } else {
-                diff = Some(CharacterStyles::new().strike(new_styles.strike));
-                self.strike = new_styles.strike;
-            }
+            diff.strike = new_styles.strike;
         }
         if self.hidden != new_styles.hidden {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.hidden(new_styles.hidden));
-                self.hidden = new_styles.hidden;
-            } else {
-                diff = Some(CharacterStyles::new().hidden(new_styles.hidden));
-                self.hidden = new_styles.hidden;
-            }
+            diff.hidden = new_styles.hidden;
         }
         if self.reverse != new_styles.reverse {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.reverse(new_styles.reverse));
-                self.reverse = new_styles.reverse;
-            } else {
-                diff = Some(CharacterStyles::new().reverse(new_styles.reverse));
-                self.reverse = new_styles.reverse;
-            }
+            diff.reverse = new_styles.reverse;
         }
         if self.slow_blink != new_styles.slow_blink {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.blink_slow(new_styles.slow_blink));
-                self.slow_blink = new_styles.slow_blink;
-            } else {
-                diff = Some(CharacterStyles::new().blink_slow(new_styles.slow_blink));
-                self.slow_blink = new_styles.slow_blink;
-            }
+            diff.slow_blink = new_styles.slow_blink;
         }
         if self.fast_blink != new_styles.fast_blink {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.blink_fast(new_styles.fast_blink));
-                self.fast_blink = new_styles.fast_blink;
-            } else {
-                diff = Some(CharacterStyles::new().blink_fast(new_styles.fast_blink));
-                self.fast_blink = new_styles.fast_blink;
-            }
+            diff.fast_blink = new_styles.fast_blink;
         }
         if self.underline != new_styles.underline {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.underline(new_styles.underline));
-                self.underline = new_styles.underline;
-            } else {
-                diff = Some(CharacterStyles::new().underline(new_styles.underline));
-                self.underline = new_styles.underline;
-            }
+            diff.underline = new_styles.underline;
         }
         if self.bold != new_styles.bold {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.bold(new_styles.bold));
-                self.bold = new_styles.bold;
-            } else {
-                diff = Some(CharacterStyles::new().bold(new_styles.bold));
-                self.bold = new_styles.bold;
-            }
+            diff.bold = new_styles.bold;
         }
         if self.dim != new_styles.dim {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.dim(new_styles.dim));
-                self.dim = new_styles.dim;
-            } else {
-                diff = Some(CharacterStyles::new().dim(new_styles.dim));
-                self.dim = new_styles.dim;
-            }
+            diff.dim = new_styles.dim;
         }
         if self.italic != new_styles.italic {
-            if let Some(new_diff) = diff.as_mut() {
-                diff = Some(new_diff.italic(new_styles.italic));
-                self.italic = new_styles.italic;
-            } else {
-                diff = Some(CharacterStyles::new().italic(new_styles.italic));
-                self.italic = new_styles.italic;
-            }
+            diff.italic = new_styles.italic;
+        }
+        if self.link_anchor != new_styles.link_anchor {
+            diff.link_anchor = new_styles.link_anchor;
         }
 
+        // apply new styles
+        *self = *new_styles;
+
         if let Some(changed_colors) = changed_colors {
-            if let Some(AnsiCode::ColorIndex(color_index)) = diff.and_then(|diff| diff.foreground) {
+            if let Some(AnsiCode::ColorIndex(color_index)) = diff.foreground {
                 if let Some(changed_color) = changed_colors[color_index as usize] {
-                    diff.as_mut().unwrap().foreground = Some(changed_color);
+                    diff.foreground = Some(changed_color);
                 }
             }
-            if let Some(AnsiCode::ColorIndex(color_index)) = diff.and_then(|diff| diff.background) {
+            if let Some(AnsiCode::ColorIndex(color_index)) = diff.background {
                 if let Some(changed_color) = changed_colors[color_index as usize] {
-                    diff.as_mut().unwrap().background = Some(changed_color);
+                    diff.background = Some(changed_color);
                 }
             }
         }
-        diff
+        Some(diff)
     }
     pub fn reset_all(&mut self) {
         self.foreground = Some(AnsiCode::Reset);
@@ -633,6 +564,12 @@ impl Display for CharacterStyles {
     }
 }
 
+#[derive(Clone, Copy, Debug, PartialEq)]
+pub enum LinkAnchor {
+    Start(u16),
+    End,
+}
+
 #[derive(Clone, Copy, Debug)]
 pub enum CharsetIndex {
     G0,
@@ -748,7 +685,7 @@ impl Cursor {
             x,
             y,
             is_hidden: false,
-            pending_styles: CharacterStyles::new(),
+            pending_styles: RESET_STYLES,
             charsets: Default::default(),
             shape: CursorShape::Initial,
         }
@@ -761,13 +698,23 @@ impl Cursor {
     }
 }
 
-#[derive(Clone, Copy)]
+#[derive(Clone, Copy, PartialEq)]
 pub struct TerminalCharacter {
     pub character: char,
     pub styles: CharacterStyles,
     pub width: usize,
 }
 
+impl TerminalCharacter {
+    pub fn new(character: char) -> Self {
+        TerminalCharacter {
+            character,
+            styles: CharacterStyles::default(),
+            width: character.width().unwrap_or(0),
+        }
+    }
+}
+
 impl ::std::fmt::Debug for TerminalCharacter {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         write!(f, "{}", self.character)
